"""
For parsing geosupport results. See the Character-Only Work Area Layouts section starting on page 566
of the Geosupport User Programming Guide for more information.
"""


def parseWA1(wa1):
    """
    :param wa1: Processed from all functions
    :return: Dictionary of results
    """
    return {
        'First Borough Name': wa1[360:369],
        'House Number - Display Format': wa1[369: 385],
        'House Number - Sort Format': wa1[385: 396],
        'B10SC - First Borough and Street Code': wa1[396: 407],
        'First Street Name Normalized': wa1[407:439],
        'B10SC - Second Borough and Street Code': wa1[439:449],
        'Second Street Name Normalized': wa1[450:481],
        'B10SC - Third Borough and Street Code': wa1[482:492],
        'Third Street Name Normalized': wa1[493:524],
        # 'BOROUGH BLOCK LOT': {'Borough Code': wa1[525:525],
        #                       'Tax Block': wa1[526:530],
        #                       'Tax Lot': wa1[531:534]},
        # 'Filler for Tax Lot Version Number': wa1[535:535],
        'Low House Number - Display Format': wa1[536:551],
        'Low House Number - Sort Format': wa1[552:562],
        'BIN': wa1[563:569],  # Building Identification Number
        'Street Attribute Indicators': wa1[570:572],
        'Reason Code 2': wa1[573:573],
        'Reason Code Qualifier 2': wa1[574],
        'Warning Code 2': wa1[575:576],
        'GRC 2': wa1[577:578],  # Geosupport Return Code 2
        'Message 2': wa1[579:658],
        'Node Number': wa1[659:665],
        # 'Filler': wa1[666:704],
        'NIN': wa1[705:710],  # NAP Identification Number
        'Street Attribute Indicator': wa1[711],
        'Reason Code': wa1[712],
        'Reason Code Qualifier': wa1[713],
        'Warning Code': wa1[714:715],
        'GRC': wa1[716:717],  # Geosupport Return Code
        'Message': wa1[718:797],
        'Number of Street Codes and Street Names in List': wa1[798:799],
        'List of Street Codes': wa1[800:879],
        'List of Street Names': wa1[880:1199],
    }


def parseWA2_1B(wa2):
    """
    Parses results of WA2 from function 1B (Street Address)
    See page 586 of the Geosupport User Programming Guide for more information.
    :param wa2: Work Area 2 from GeoSupport function 1B results
    :return: Dictionary of results
    """
    return {
        'Continuous Parity Indicator/Duplicate Address Indicator': wa2[21:21],
        'Low House Number of Block Face': wa2[22:32],
        'High House Number of Block Face': wa2[33:43],
        'DCP Preferred LGC': wa2[44:45],
        'Number of Cross Streets at Low Address End': wa2[46:46],
        'List of Cross Streets at Low Address End (Up to 5 B5SCs)': wa2[47:76],
        'Number of Cross Streets at High Address End': wa2[77:77],
        'List of Cross Streets at High Address End (Up to 5 B5SCs)': wa2[78:107],
        'LION KEY Borough Code': wa2[108],
        'LION KEY Face Code': wa2[109:112],
        'LION KEY Sequence Number': wa2[113:117],
        'Special Address Generated Record Flag': wa2[118],
        'Side of Street Indicator': wa2[119],
        'Segment Length in Feet': wa2[120:124],
        'XCoord': wa2[125:131],
        'YCoord': wa2[132:138],
        'Reserved for Possible Z Coordinate': wa2[139:145],
        'Community Development Eligibility Indicator': wa2[146],
        'Marble Hill/Rikers Island Alternative Borough Flag': wa2[147],
        'DOT Street Light Contractor Area': wa2[148],
        'Community District Borough Code': wa2[149],
        'Community District Number': wa2[150:151],
        'Zip Code': wa2[152:157],
        'Election District': wa2[158:159],
        'Assembly District': wa2[159:161],
        'Split Election District Flag': wa2[162],
        'Congressional District': wa2[163:164],
        'State Senatorial District': wa2[165:166],
        'Civil Court District': wa2[167:168],
        'City Council District': wa2[169:170],
        'Health Center District': wa2[171:172],
        'Health Area': wa2[173:176],
        'Sanitation District': wa2[177:179],
        'Sanitation Collection Scheduling Section and Subsection': wa2[180:181],
        'Sanitation Regular Collection Schedule': wa2[182:186],
        'Sanitation Recycling Collection Schedule': wa2[187:189],
        'Police Patrol Borough Command': wa2[190],
        'Police Precinct': wa2[191:194],
        'Fire Division': wa2[195:195],
        'Fire Battalion': wa2[196:197],
        'Fire Company Type': wa2[198],
        'Fire Company Number': wa2[199:201],
        # 'Filler': wa2[202:202],
        'Community School District': wa2[203:204],
        'Atomic Polygon': wa2[205:207],
        'Police Patrol Borough': wa2[208:209],
        'Feature Type Code': wa2[210],
        'Segment Type Code': wa2[211],
        'Alley or Cross Street List Flag': wa2[212],
        'Coincidence Segment Count': wa2[213],
        # 'Filler': wa2[214:216],
        '1990 Census Tract': wa2[217:222],
        '2010 Census Tract': wa2[223:228],
        '2010 Census Block': wa2[229:232],
        '2010 Census Block Suffix': wa2[233],
        '2000 Census Tract': wa2[234:239],
        '2000 Census Block': wa2[240:243],
        '2000 Census Block Suffix': wa2[244],
        'Neighborhood Tabulation Area (NTA)': wa2[245:248],
        'DSNY Snow Priority Code': wa2[249],
        'DSNY Organic Recycling Schedule': wa2[250:254],
        'DSNY Reserved': wa2[255:259],
        'Hurricane Evacuation Zone (HEZ)': wa2[260:261],
        # 'Filler': wa2[262:272],
        'Underlying Address Number for NAPs': wa2[273:283],
        'Underlying B7SC': wa2[284:291],
        'Segment Identifier': wa2[292:298],
        'Curve Flag': wa2[301],
        "List of 4 LGC's": wa2[302:307],
        'BOE LGC Pointer': wa2[308],
        'Segment Azimuth': wa2[309:311],
        'Segment Orientation': wa2[312],
        'Segment XCoord, Low Address End': wa2[313:319],
        'Segment Y Coord, Low Address End': wa2[320:326],
        'Segment Z Coord, Low Address End': wa2[327:333],
        'Segment X Coord, High Address End': wa2[334:340],
        'Segment Y Coord, High Address End': wa2[341:347],
        'Segment Z Coord, High Address End': wa2[348:351],
        'Roadway Type': wa2[444:446],
        'Bike Lane': wa2[486],
        'NTA Name': wa2[553: 628],
        'USPS Preferred City Name': wa2[628:653],
        'Latitude': wa2[653:662],
        'Longitude': wa2[662:672],
        'Borough code': wa2[1533],
        'Tax Block': wa2[1534:1539],
        'Tax Lot': wa2[1539:1543],
        'Building Identification Number (BIN) of Input Address or NAP': wa2[1581:1588],
        'X-Y Coordinates of Lot Centroid': wa2[1699:1713],
    }


def parseWA2_1A_BL_BN(wa2):
    """
    Parses results of WA2 from function 1A, BL, BN.
    Property Level Information Defined by Address, BBL or BIN.
    See page 579 of Geosupport User Programming Guide for more info.

    :param wa1: Work Area 1 from GeoSupport function BL results
    :param wa2: Work Area 2 from GeoSupport function BL results
    :return: Dictionary of results
    """

    return {
        'Continuous Parity Indicator /Duplicate Address Indicator': wa2[21],
        'Low House Number of Defining Address Range': wa2[22:32],
        'Borough code': wa2[33],
        'Tax Block': wa2[34:38],
        'Tax Lot': wa2[36:42],
        # 'Filler for Tax Lot Version Number': wa2[43:43],
        'RPAD Self-Check Code (SCC) for BBL': wa2[44],
        # 'Filler': wa2[45:45],
        'RPAD Building Classification Code': wa2[46:47],
        'Corner Code': wa2[48:49],
        'Number of Existing Structures on Lot': wa2[50:53],
        'Number of Street Frontages of Lot': wa2[54:55],
        'Interior Lot Flag': wa2[56],
        'Vacant Lot Flag': wa2[57],
        'Irregularly-Shaped Lot Flag': wa2[58],
        'Marble Hill/Rikers Island Alternate Borough Flag': wa2[59],
        'List of Geographic Identifiers (LGI) Overflow Flag': wa2[60],
        'STROLLING KEY Borough': wa2[61],
        "STROLLING KEY 5-Digit Street Code of 'On' Street": wa2[62:66],
        'STROLLING KEY Side of Street Indicator': wa2[67],
        "STROLLING KEY High House Number - Sort Format": wa2[68:78],
        # 'Filler': wa2[79:79],

        'Reserved for Internal Use': wa2[80],
        'Building Identification Number (BIN) of Input Address or NAP': wa2[81:87],
        'Condominium Flag': wa2[88],
        # 'Filler': wa2[89:89],
        'DOF Condominium Identification Number': wa2[90:93],
        'Condominium Unit ID Number': wa2[94:100],
        'Condominium Billing BBL': wa2[101:110],
        # 'Filler - Tax Lot Version No. for Billing BBL': wa2[111:11],
        'Self-Check Code (SCC) of Billing BBL': wa2[112],
        'Low BBL Condo Borough Code': wa2[113],  # LOW BBL OF THIS BUILDING'S CONDOMINIUM UNITS
        'Low BBL Condo Tax Block': wa2[114:118],  # LOW BBL OF THIS BUILDING'S CONDOMINIUM UNITS
        'Low BBL Condo Tax Lot': wa2[119:122],  # LOW BBL OF THIS BUILDING'S CONDOMINIUM UNITS
        # 'Filler for Tax Lot Version No. of Low BBL': wa2[123:123],
        'High BBL Condo Borough Code': wa2[124],  # HIGH BBL OF THIS BUILDING'S CONDOMINIUM UNITS
        'High BBL Condo Tax Block': wa2[125:129],  # HIGH BBL OF THIS BUILDING'S CONDOMINIUM UNITS
        'High BBL Condo Tax Lot': wa2[130:133],  # HIGH BBL OF THIS BUILDING'S CONDOMINIUM UNITS
        # "Filler for Tax Lot Version No. of High BBL": wa2[134:134],
        # "Filler": wa2[135:149],
        "Cooperative ID Number": wa2[150:153],
        "Sanborn Borough Code": wa2[154],  # SBVP (SANBORN MAP IDENTIFIER)
        "Volume Number": wa2[155:156],  # SBVP (SANBORN MAP IDENTIFIER)
        "Volume Number Suffix": wa2[157],  # SBVP (SANBORN MAP IDENTIFIER)
        "Page Number": wa2[158:160],  # SBVP (SANBORN MAP IDENTIFIER)
        "Page Number Suffix": wa2[161],  # SBVP (SANBORN MAP IDENTIFIER)
        "DCP Commercial Study Area": wa2[162:166],
        "Tax Map Number Section & Volume": wa2[167:171],
        "Reserved for Tax Map Page Number": wa2[172:175],
        # "Filler": wa2[176:178],
        "Latitude": wa2[179:187],
        "Longitude": wa2[188:198],
        "X-Y Coordinates of Tax Lot Centroid (Internal to Lot)": wa2[199:212],
        "Business Improvement District (BID)": wa2[213:218],
        "TPAD BIN Status (for DM job)": wa2[219],
        "TPAD New BIN": wa2[220:226],
        "TPAD New BIN Status": wa2[227],
        "TPAD Conflict Flag": wa2[228],
        # "Filler": wa2[229:237],
        "List of 4 LGCs": wa2[238:245],
        "Number of Entries in List of Geographic Identifiers": wa2[246:249],
        "LIST OF GEOGRAPHIC IDENTIFIERS": wa2[250:1362],
        # {
        # "Low House Number":
        # "High House Number":
        # "Borough Code":
        # "5-Digit Street Code":
        # "DCP-Preferred Local Group Code (LGC)":
        # "Building Identification Number (BIN)":
        # "Side of Street Indicator":
        # "Geographic Identifier Entry Type Code":
        # "TPAD BIN Status":
        # "Filler":
        # }

    }


def parseWA2_2W(wa2):
    return {
        "Intersection Replication Counter": wa2[22],
        "DCP-Preferred LGC for Street 1": wa2[24:25],
        "DCP-Preferred LGC for Street 2": wa2[26:27],
        "Number of Intersecting Streets": wa2[28],
        "List of Intersecting Streets": wa2[29:58],
        "Compass Direction for Intersection Key or Counter for Multiple Intersections": wa2[59],
        "Atomic Polygon": wa2[60:62],
        # "Filler": wa2[63:64],
        "LION Node Number": wa2[65:71],
        "XCoord": wa2[72:78],
        "YCoord": wa2[78:84],
        "ZCoord": wa2[85:91],
        "SBVP1 Borough Code": wa2[93],
        "SBVP1 Volume Number": wa2[94:95],
        "SBVP1 Volume Number Suffix": wa2[96],
        "SBVP1 Page Number": wa2[97:99],
        "SBVP1 Page Number Suffix": wa2[100],
        "SBVP2 Borough Code": wa2[101],
        "SBVP2 Volume Number": wa2[102:103],
        "SBVP2 Volume Number Suffix": wa2[104],
        "SBVP2 Page Number": wa2[105:107],
        "SBVP2 Page Number Suffix": wa2[108],
        "Marble Hill/Rikers Island Alternative Borough Flag": wa2[109],
        "DOT Street Light Contractor Area": wa2[110],
        "Community District Borough Code": wa2[111],
        "Community District Number": wa2[112:113],
        "ZIP Code": wa2[114:118],
        "Health Area": wa2[119:122],
        "Police Patrol Borough Command": wa2[123],
        "Police Precinct": wa2[124:126],
        "Fire Division": wa2[127:128],
        "Fire Battalion": wa2[129:130],
        "Fire Company Type": wa2[131],
        "Fire Company Number": wa2[132:134],
        "Community School District": wa2[135:136],
        "2010 Census Tract": wa2[137:142],
        "1990 Census Tract": wa2[134:148],
        "List of Pairs of Level Codes": wa2[149:158],
        "Police Patrol Borough": wa2[159:160],
        "Assembly District": wa2[161:162],
        "Congressional District": wa2[163:164],
        "State Senatorial District": wa2[165:166],
        "Civil Court District": wa2[167:168],
        "City Council District": wa2[169:170],
        "CD Eligibility": wa2[171],
        "Distance Between Duplicate Intersections": wa2[172:176],
        "2000 Census Tract": wa2[177:182],
        "Health Center District": wa2[183:184],
        "Sanitation District": wa2[185:187],
        "Sanitation Section/Subsection": wa2[188:189],
        # "Filler": wa2[24:25],

        # 2W
        "LGC List for Street 1": wa2[224:231],
        "LGC List for Street 2": wa2[232:239],
        "Turn Restrictions": wa2[240:249],
        "Preferred LGCs for Intersecting B5SCs": wa2[250:259],
        "True Replication Counter": wa2[260:261],
        "List of Up To 20 7-Byte Nodes": wa2[262:401],
        "List of intersecting streets (B7SCs) for node list": wa2[402:3601],
        "Reason Code": wa2[3602],
        "Reason Code Qualifier": wa2[3603],
        "Warning Code": wa2[3604:3605],
        "Return Code": wa2[3606:3607],
        "Latitude": wa2[3606:3615],
        "Longitude": wa2[3615:3625]
    }


# def parseWA2_3(wa2):
#     return {'Duplicate Key Flag or Continuous Parity': wa2[21],
#             'Locational Status of Segment': wa2[22],
#             'County Boundary Indicator': wa2[23],
#             'DCP-Preferred LGC for Street 1': wa2[24:25],
#             'DCP-Preferred LGC for Street 2': wa2[],
#             'DCP-Preferred LGC for Street 3': wa2[],
#             'Number of Cross Streets at Low Address End': wa2[],
#             'List of Cross Streets at Low Address End': wa2[],
#             'Number of Cross Streets at High Address End': wa2[],
#             'List of Cross Streets at High Address End': wa2[],
#             'Cross Street Reversal Flag': wa2[],
#             'LION Borough Code': wa2[],
#             'LION Face Code': wa2[],
#             'LION Sequence Number': wa2[],
#             'Generated Record Flag': wa2[],
#             'Length of Segment in Feet': wa2[],
#             'Segment Azimuth': wa2[],
#             'Segment Orientation': wa2[],
#             'Marble Hill/Rikers Island Alternative Borough Flag': wa2[],
#             'From Node': wa2[],
#             'To Node': wa2[],
#             'DSNY Snow Priority Code': wa2[],
#             # 'Filler: wa2[],
#             'Segment Identifier': wa2[],
#             'DOT Street Light Contractor Area': wa2[],
#             'Curve Flag': wa2[],
#             'Dog Leg Flag': wa2[],
#             'Feature Type Code': wa2[],
#             'Segment Type Code': wa2[],
#             'Coincident Segment Count': wa2[],
#             # 'Filler': wa2[],
#
#             'Left Side': {
#                 'Community District Borough Code': wa2[],
#                 'Community District Number': wa2[],
#                 'Low House Number': wa2[],
#                 'High House Number': wa2[],
#                 'Future Use': wa2[],
#                 'Community Development Eligibility Indicator': wa2[],
#                 'ZIP Code': wa2[],
#                 'Health Area': wa2[],
#                 'Police Patrol Borough Command': wa2[],
#                 'Police Precinct': wa2[],
#                 'Fire Division': wa2[],
#                 'Fire Battalion': wa2[],
#                 'Fire Company Type': wa2[],
#                 'Fire Company Number': wa2[],
#                 'Community School District': wa2[],
#                 'Atomic Polygon': wa2[],
#                 'Election District (ED)': wa2[],
#                 'Assembly District (AD)': wa2[],
#                 'Police Patrol Borough': wa2[],
#                 # 'Filler': wa2[],
#                 'Borough Code': wa2[],
#                 '1990 Census Tract': wa2[],
#                 '2010 Census Tract': wa2[],
#                 '2010 Census Block': wa2[],
#                 '2010 Census Block Suffix': wa2[],
#                 '2000 Census Tract': wa2[],
#                 '2000 Census Block': wa2[],
#                 '2000 Census Block Suffix': wa2[],
#                 'Neighborhood Tabulation Area (NTA)': wa2[],
#                 # 'Filler': wa2[],
#             },
#
#             'Right Side': {
#                 'Community District Borough Code': wa2[],
#                 'Community District Number': wa2[],
#                 'Low House Number': wa2[],
#                 'High House Number': wa2[],
#                 'Future Use': wa2[],
#                 'Community Development Eligibility Indicator': wa2[],
#                 'ZIP Code': wa2[],
#                 'Health Area': wa2[],
#                 'Police Patrol Borough Command': wa2[],
#                 'Police Precinct': wa2[],
#                 'Fire Division': wa2[],
#                 'Fire Battalion': wa2[],
#                 'Fire Company Type': wa2[],
#                 'Fire Company Number': wa2[],
#                 'Community School District': wa2[],
#                 'Atomic Polygon': wa2[],
#                 'Election District (ED)': wa2[],
#                 'Assembly District (AD)': wa2[],
#                 'Police Patrol Borough': wa2[],
#                 # 'Filler': wa2[],
#                 'Borough Code': wa2[],
#                 '1990 Census Tract': wa2[],
#                 '2010 Census Tract': wa2[],
#                 '2010 Census Block': wa2[],
#                 '2010 Census Block Suffix': wa2[],
#                 '2000 Census Tract': wa2[],
#                 '2000 Census Block': wa2[],
#                 '2000 Census Block Suffix': wa2[],
#                 # 'Filler': wa2[],
#                 'Neighborhood Tabulation Area (NTA)': wa2[],
#                 # 'Filler': wa2[],
#             },
#
#             # Function 3 Extended:
#             'List of 4 LGCs for Street 1': wa2[],
#             'List of 4 LGCs for Street 2': wa2[],
#             'List of 4 LGCs for Street 3': wa2[],
#             'Left Health Center District': wa2[],
#             'Right Health Center District': wa2[],
#             # 'Filler': wa2[],
#             'Traffic Direction': wa2[],
#             'Roadway Type': wa2[],
#             'Physical ID': wa2[],
#             'Generic ID': wa2[],
#             'NYPD ID': wa2[],
#             'FDNY ID': wa2[],
#             'Street Status': wa2[],
#             'Street Width': wa2[],
#             'Street Width Irregular': wa2[],
#             'Bike Lane': wa2[],
#             'Federal Classification Code': wa2[],
#             'Right of Way Type': wa2[],
#             'List of 5 Additional LGCs for Street 1': wa2[],
#             'Legacy ID': wa2[],
#             'Left NTA Name': wa2[],
#             'Right NTA Name': wa2[],
#             'From X Coordinate': wa2[],
#             'From Y Coordinate': wa2[]
#             'To X Coordinate': wa2[],
#             'To Y Coordinate': wa2[],
#             'Latitude of From Intersection': wa2[],
#             'Longitude of From Intersection': wa2[],
#             'Latitude of To Intersection': wa2[],
#             'Longitude of To Intersection': wa2[],
#             'Left Blockface ID': wa2[],
#             'Right Blockface ID': wa2[],
#             'Number of Travel Lanes on the Street': wa2[],
#             'Number of Parking Lanes on the Street': wa2[],
#             'Number of Total Lanes on the Street': wa2[],
#             # 'Filler': wa2[],
#             '': wa2[],
#             '': wa2[],
#             '': wa2[],
#             }
